@@ -13,15 +13,16 @@ from pstats import Stats
 from jinja2 import Template
 
 
-__version__ = '0.1.3'
-__author__  = 'Hideo Hattori <syobosyobo@gmail.com>'
+__version__ = '0.1.4'
 __licence__ = 'New BSD License'
+__author__ = 'Hideo Hattori <syobosyobo@gmail.com>'
 
 __all__ = ['Converter']
 
 XHTML_HEADER_TEMPLATE = u"""\
 <?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
+ "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
 <head>
 
@@ -46,23 +47,31 @@ table.prof { border-collapse: collapse; border: 1px #cccccc solid; }
 td.prof { border-collapse: collapse; border: 1px #cccccc solid; }
 span.tdmg { margin-left: 5px; margin-right: 5px; }
 pre.tdms {
-  margin-left: 5px;
-  margin-right: 5px;
-  margin-top: 0px;
-  margin-bottom: 0px;
-  font-family: courier new;
-  padding: 0px;
+    margin-left: 5px;
+    margin-right: 5px;
+    margin-top: 0px;
+    margin-bottom: 0px;
+    font-family: courier new;
+    padding: 0px;
 }
 th.prof {
     border-collapse: collapse;
     border: 1px #cccccc solid;
     background-color: #eae0d5;
 }
-td.profline {
+td.profline_r {
     border-collapse: collapse;
     border: 1px #cccccc solid;
     font-size: 75%;
-    padding:0px;
+    padding:0px 1px 0px 0px;
+    text-align: right;
+}
+td.profline_l {
+    border-collapse: collapse;
+    border: 1px #cccccc solid;
+    font-size: 75%;
+    padding:0px 1px 0px 0px;
+    text-align: left;
 }
 th.procheader { align: center; background: #a294c8; }
 #sidebar {
@@ -71,10 +80,10 @@ th.procheader { align: center; background: #a294c8; }
     padding-left:10px;
     padding-right:10px;
     right: 0;
-    top  : 0;
+    top: 0;
     width: 160px;
 }
-#content { margin-left : 1em; margin-right: 1em; text-align: left; }
+#content { margin-left: 1em; margin-right: 1em; text-align: left; }
 #footer {
     clear     : both;
     text-align: center;
@@ -85,6 +94,9 @@ th.procheader { align: center; background: #a294c8; }
     background-color: #eae0d5;
 }
 div.profilelinetable { margin-top: 1em; }
+div.header a { color: #000000; }
+div.header a:visited { color: #000000; }
+div.header a:hover { color: #000000; }
 
 </style>
 """
@@ -95,7 +107,8 @@ XHTML_BODY_TEMPLATE = u"""\
 
 <body>
     <div class="header">
-        <span class="header-title">Profile Report</span>
+        <span class="header-title"><a href="./index.html"
+        >Profile Report</a></span>
         <span class="header-time">Profile Time on {{ proftime }}</span><br/>
         <span class="header-time">Reported Time on {{ reporttime }}</span>
     </div>
@@ -107,47 +120,85 @@ XHTML_BODY_TEMPLATE = u"""\
     <p>Total Time     : <span style="font-weight:bold"
                         >{{ profdata['totaltime'] }}</span> CPU second<br/>
        Total Func Call: <span style="font-weight:bold"
-                        >{{ profdata['totalcalls'] }}</span> function calls</p>
+                        >{{ profdata['totalcalls'] }}</span
+                        > function calls</p>
     <table class="prof">
         <tr>
             <th class="procheader" align="center" colspan="6"
-            >Top {{ profdata['data']|length }} functions data (<a href="
+            >Top {{ profdata['data']|length }} functions data &#40;<a href="
             {% if thisname == 'index.html' %}
-                index-all.html">all function)</th>
+                index-all.html">all function</a>&#41;</th>
             {% else %}
-                index.html">top 20 function</a>)</th>
+                index.html">top 20 function</a>&#41;</th>
             {% endif %}
         </tr>
         <tr align="right">
-            <th class="prof"><span class="tdmg" title="for the number of calls">ncalls</span></th>
-            <th class="prof"><span class="tdmg" title="for the total time spent in the given function (and excluding time made in calls to sub-functions)">tottime</span></th>
-            <th class="prof"><span class="tdmg" title="the quotient of tottime divided by ncalls">percall</span></th>
-            <th class="prof"><span class="tdmg" title="total time spent in this and all subfunctions (from invocation till exit). This figure is accurate even for recursive functions.">cumtime</span></th>
-            <th class="prof"><span class="tdmg" title="the quotient of cumtime divided by primitive calls">percall</span></th>
-            <th class="prof" align="center"><span class="tdmg" title="provides the respective data of each function">filename:lineno(function)</span></th>
+        <th class="prof"><span class="tdmg"
+                               title="for the number of calls"
+                               >ncalls</span></th>
+        <th class="prof"
+        ><span class="tdmg" title="for the total time spent in the given
+ function (and excluding time made in calls to sub-functions)"
+        >tottime</span></th>
+        <th class="prof"
+        ><span class="tdmg" title="the quotient of tottime divided by ncalls"
+        >percall</span></th>
+        <th class="prof"
+        ><span class="tdmg" title="total time spent in this and all
+ subfunctions (from invocation till exit).
+ This figure is accurate even for recursive functions.">cumtime</span></th>
+        <th class="prof"
+        ><span class="tdmg"
+        title="the quotient of cumtime divided by primitive calls"
+        >percall</span></th>
+        <th class="prof" align="center"
+        ><span class="tdmg"
+               title="provides the respective data of each function"
+        >filename:lineno(function)</span></th>
         </tr>
         {% for item in profdata['data'] %}
             <tr>
-            <td class="prof" style="background-color:{{ item['ncallslevel'] }}" align="right">
+            <td class="prof"
+                style="background-color:{{ item['ncallslevel'] }}"
+                align="right">
               <span class="tdmg">{{ item['ncalls'] }}</span></td>
-            <td class="prof" style="background-color:{{ item['tottimelevel'] }}" align="right">
+            <td class="prof"
+                style="background-color:{{ item['tottimelevel'] }}"
+                align="right">
               <span class="tdmg">{{ item['tottime'] }}</span></td>
-            <td class="prof" style="background-color:{{ item['totcalllevel'] }}" align="right">
+            <td class="prof"
+                style="background-color:{{ item['totcalllevel'] }}"
+                align="right">
               <span class="tdmg">{{ item['totpercall'] }}</span></td>
-            <td class="prof" style="background-color:{{ item['cumtimelevel'] }}" align="right">
+            <td class="prof"
+                style="background-color:{{ item['cumtimelevel'] }}"
+                align="right">
               <span class="tdmg">{{ item['cumtime'] }}</span></td>
-            <td class="prof" style="background-color:{{ item['cumcalllevel'] }}" align="right">
+            <td class="prof"
+                style="background-color:{{ item['cumcalllevel'] }}"
+                align="right">
               <span class="tdmg">{{ item['cumpercall'] }}</span></td>
             {% if item['func'][0] == '~' %}
-                <td class="prof"><span class="tdmg">{{ item['func'][2]|escape }}</span></td>
+                <td class="prof"
+                 ><span class="tdmg">{{ item['func'][2]|escape }}</span></td>
             {% else %}
                 <td class="prof"><span class="tdmg"
                 {% if proftype == 'line' %}
-                    ><a href="{{ item['linelink'] }}">{{ item['func'][0]|escape }}</a> : 
+                    {% if item['linelink'][0] != "<" %}
+                        ><a href="{{ item['linelink']|escape }}"
+                        >{{ item['func'][0]|escape }}</a
+                        > : {{ item['func'][1] }} &#40;
+                         {{ item['func'][2]|escape }}&nbsp;&#41;</span></td>
+                    {% else %}
+                        >{{ item['func'][0]|escape }} :
+                         {{ item['func'][1] }} &#40;
+                         {{ item['func'][2]|escape }}&nbsp;&#41;</span></td>
+                    {% endif %}
                 {% else %}
-                    >{{ item['func'][0]|escape }}</a> : 
+                    >{{ item['func'][0]|escape }} :
+                     {{ item['func'][1] }} &#40;
+                     {{ item['func'][2]|escape }}&nbsp;&#41;</span></td>
                 {% endif %}
-                {{ item['func'][1] }} ({{ item['func'][2]|escape }})</span></td>
             {% endif %}
             </tr>
         {% endfor %}
@@ -160,7 +211,8 @@ XHTML_LINEBODY_TEMPLATE = u"""\
 
 <body>
     <div class="header">
-        <span class="header-title">Profile Report</span>
+        <span class="header-title"><a href="./index.html"
+        >Profile Report</a></span>
         <span class="header-time">Profile Time on {{ proftime }}</span><br/>
         <span class="header-time">Reported Time on {{ reporttime }}</span>
     </div>
@@ -172,34 +224,40 @@ XHTML_LINEBODY_TEMPLATE = u"""\
     <p>Total Time     : <span style="font-weight:bold"
                         >{{ profdata['totaltime'] }}</span> CPU second<br/>
        Total Func Call: <span style="font-weight:bold"
-                        >{{ profdata['totalcalls'] }}</span> function calls</p>
+                        >{{ profdata['totalcalls'] }}</span
+                        > function calls</p>
     <div class="profilelinetable">
     <table class="prof">
     <tr>
-    <th class="procheader" align="center" colspan="3">{{ filename }}</th>
+    <th class="procheader" align="center" colspan="4">{{ filename }}</th>
     </tr>
     <tr align="right">
-    <th class="prof" align="center"><span class="tdmg"
-     >&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;time&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span></th>
     <th class="prof" align="center"><span class="tdmg">num</span></th>
+    <th class="prof" align="center"><span class="tdmg"
+     >&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;time&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span
+     ></th>
+    <th class="prof" align="center"><span class="tdmg">calls</span></th>
     <th class="prof" align="center"><span class="tdmg">code</span></th>
     </tr>
     {% for item in profdata %}
         <tr>
+        <td class="profline_r"><span class="tdms" style="color:#999999;"
+        >{{ loop.index }}</span></td>
         {% if item['sec'] == 'None' %}
-            <td class="profline" align="right"><span class="tdms"></span></td>
+            <td class="profline_r"><span class="tdms"></span></td>
+            <td class="profline_r"><span class="tdms"></span></td>
         {% else %}
-            <td class="profline" align="right" style="background-color:#ffffff;"
-             ><span class="tdms">{{ item['sec'] }} sec</span></td>
+            <td class="profline_r" style="background-color:#ffffff;"
+            ><span class="tdms">{{ item['sec'] }} sec</span></td>
+            <td class="profline_r" style="background-color:#ffffff;"
+            ><span class="tdms">{{ item['cnt'] }} call</span></td>
         {% endif %}
-        <td class="profline" align="right"><span class="tdms" style="color:#999999;"
-                                            >{{ loop.index }}</span></td>
         {% if item['sec'] == 'None' %}
-            <td class="profline" align="left"><pre class="tdms"
-                                               >{{ item['line']|escape }}</pre></td>
+            <td class="profline_l"
+             ><pre class="tdms">{{ item['line']|escape }}</pre></td>
         {% else %}
-            <td class="profline" align="left"><pre class="tdms" style="font-weight:bold;"
-                                               >{{ item['line']|escape }}</pre></td>
+            <td class="profline_l"><pre class="tdms" style="font-weight:bold;"
+             >{{ item['line']|escape }}</pre></td>
         {% endif %}
         </tr>
     {% endfor %}
@@ -211,8 +269,8 @@ XHTML_FOOTER_TEMPLATE = """\
     </div>
     <div id="footer">
     <p>
-    Powered by <a href="http://www.python.org/">Python</a> and 
-    <a href="http://jinja.pocoo.org/2/">Jinja2</a>&nbsp;&nbsp;&nbsp;
+    Powered by <a href="http://www.python.org/">Python</a
+    > and <a href="http://jinja.pocoo.org/2/">Jinja2</a>&nbsp;&nbsp;&nbsp;
     <a href="http://pypi.python.org/pypi/pyprof2html/">pyprof2html</a>: %s
     </p>
     </div>
@@ -225,21 +283,30 @@ def check_hotlinetimings(dump):
     signature = "yes"
     return signature in dump
 
+
 def check_hotshot(dump):
     """check to hotshot-profile datafile."""
     signature = "hotshot-version"
     return signature in dump
 
+
 def mapping_table(target, all):
+    levelmap = {0: '#ccffcc',   ## yellow green (light)
+                1: '#66ff99',   ## yellow green
+                2: '#ffcc33',   ## orange
+                3: '#ff6666',   ## pink
+                4: '#ff3333',   ## red
+                }
     level_list = [all*0.01, all*0.03, all*0.08, all*0.12, all*0.20]
     for level in range(len(level_list)):
         if target < level_list[level]:
             break
-    return level
+    return levelmap[level]
 
 
 class Converter:
-    """output to HTML from profile data"""
+    """output to HTML from profile data
+    """
 
     def __init__(self, filename):
         self.outfile = None
@@ -262,15 +329,11 @@ class Converter:
         self.reporttime = time.ctime()
         self.outputtype = 'html'
         self.functions_number = 20
-        self.levelmap = {
-            0: '#ccffcc',   ## yellow green (light)
-            1: '#66ff99',   ## yellow green
-            2: '#ffcc33',   ## orange
-            3: '#ff6666',   ## pink
-            4: '#ff3333',   ## red
-        }
 
-    def _printhtml_onesource(self, filename, profinfo):
+    def _printhtml_onesource(self, filename, profs):
+        """printing one file profile line. return value is html render
+        strings.
+        """
         num = 0
         result = []
         try:
@@ -286,8 +349,9 @@ class Converter:
             except UnicodeDecodeError:
                 line = "Unicode Decoding Error!!!"
             num += 1
-            if profinfo.has_key(num):
-                result.append({'sec':profinfo[num], 'line':line})
+            if num in profs:
+                result.append({'sec':profs[num]['sec'],
+                               'cnt':profs[num]['cnt'], 'line':line})
             else:
                 result.append({'sec':'None', 'line':line})
         titletext = "pyprof2html - %s" % filename
@@ -298,7 +362,9 @@ class Converter:
                                     profdata=result,
                                     codec=codec)
 
-    def _print_onesource(self, filename, profinfo):
+    def _print_onesource(self, filename, profs):
+        """printing one file profile line. return value is text strings.
+        """
         num = 0
         result = []
         try:
@@ -310,18 +376,22 @@ class Converter:
         result.append("="*60 + "\n")
         for line in lines:
             num += 1
-            if profinfo.has_key(num):
-                result.append(" %3.4lfsec | %s" % (profinfo[num], line))
+            if num in profs:
+                result.append(" %3.4lfs | %7dn | %s" % (profs[num]['sec'],
+                                                        profs[num]['cnt'],
+                                                        line))
             else:
-                result.append("           | %s" % (line))
+                result.append("         |          | %s" % (line))
         return ''.join(result)
 
-    def _print_source(self, profinfo):
+    def _print_source(self, profs):
+        """wrapper of _print_onesource() and _printhtml_onesource() method.
+        """
         source_list = []
         info = {}
         filename = None
-        filecnt  = 0
-        for prof in profinfo:
+        filecnt = 0
+        for prof in profs:
             filename = prof[0]
             if not source_list.count(filename):
                 if info != {}:
@@ -338,13 +408,15 @@ class Converter:
                         outputfile.close()
                     else:
                         print self._print_onesource(source_list[filecnt-1],
-                                                     info)
+                                                    info)
                 info = {}
                 source_list.append(filename)
-            info[prof[1]] = prof[2]
+            info[prof[1]] = {'sec':prof[2], 'cnt':prof[3]}
         if self.outputtype == 'html':
-            output_htmlfile = "%s.html" % os.path.basename(source_list[filecnt-1])
-            outputfile = open("%s/%s" % (self.output_dir, output_htmlfile), 'w')
+            output_htmlfile = "%s.html" % (os.path.basename(
+                                           source_list[filecnt-1]))
+            outputfile = open("%s/%s" % (self.output_dir,
+                                         output_htmlfile), 'w')
             print >> outputfile, XHTML_HEADER_TEMPLATE
             print >> outputfile, self._printhtml_onesource(filename, info)
             print >> outputfile, XHTML_FOOTER_TEMPLATE
@@ -353,17 +425,22 @@ class Converter:
             print self._print_onesource(filename, info)
 
     def _analyze_profline(self):
+        """analyzed to hotshot linetimings data.
+        """
         profset = dict()
         for i in self.profline:
-            if not profset.has_key(i[1]):
-                profset[i[1]] = float(int(i[2])/1000000.)
+            if i[1] not in profset:
+                profset[i[1]] = [float(int(i[2])/1000000.), i[2]]
             else:
-                profset[i[1]] += float(int(i[2])/1000000.)
-        proflist = [(prof[0], prof[1], profset[prof]) for prof in profset]
-        proflist.sort()
-        return proflist
+                profset[i[1]][0] += float(int(i[2])/1000000.)
+                profset[i[1]][1] += i[2]
+        profs = [(p[0], p[1], profset[p][0], profset[p][1]) for p in profset]
+        profs.sort()
+        return profs
 
     def _analyzed_prof(self):
+        """analyzed to not linetimings profile data.
+        """
         self.prof.sort_stats('time', 'calls')
         null = open('/dev/null', 'w')   # darty hack (order output block)
         self.prof.stream = null
@@ -392,39 +469,37 @@ class Converter:
                 cumpercall = "%8.4lf" % float(stat[3]/stat[0])
             except ZeroDivisionError:
                 cumpercall = "0.0000"
-            ncallslevel = self.levelmap[mapping_table(ncalls,
-                                                      self.prof.total_calls)]
-            tottlevel   = self.levelmap[mapping_table(stat[2],
-                                                      self.prof.total_tt)]
-            totclevel   = self.levelmap[mapping_table(float(totpercall),
-                                                      self.prof.total_tt)]
-            cumtlevel   = self.levelmap[0]
-            cumclevel   = self.levelmap[0]
-            linelink    = "%s.html" % os.path.basename(func[0])
+            ncallslevel = mapping_table(ncalls, self.prof.total_calls)
+            tottlevel = mapping_table(stat[2], self.prof.total_tt)
+            totclevel = mapping_table(float(totpercall), self.prof.total_tt)
+            cumtlevel = mapping_table(1, 1000)
+            cumclevel = mapping_table(1, 1000)
+            linelink = "%s.html" % os.path.basename(func[0])
             data = {
-                'ncalls'        : ncalls,
-                'tottime'       : tottime,
-                'cumtime'       : cumtime,
-                'totpercall'    : totpercall,
-                'cumpercall'    : cumpercall,
-                'ncallslevel'   : ncallslevel,
-                'cumtimelevel'  : cumtlevel,
-                'tottimelevel'  : tottlevel,
-                'totcalllevel'  : totclevel,
-                'cumcalllevel'  : cumclevel,
-                'func'          : func,
-                'linelink'      : linelink,
+                'func': func,
+                'linelink': linelink,
+                'ncalls': ncalls,
+                'tottime': tottime,
+                'cumtime': cumtime,
+                'totpercall': totpercall,
+                'cumpercall': cumpercall,
+                'ncallslevel': ncallslevel,
+                'cumtimelevel': cumtlevel,
+                'tottimelevel': tottlevel,
+                'totcalllevel': totclevel,
+                'cumcalllevel': cumclevel,
             }
             datalist.append(data)
         profdata = {
             'totaltime': "%8.4lf" % self.prof.total_tt,
             'totalcalls': self.prof.total_calls,
-            'data': datalist
+            'data': datalist,
         }
         return profdata
 
     def _hookraw(self):
-        """hook rawtext print"""
+        """hook rawtext print
+        """
         if self.proftype == 'line':
             proflist = self._analyze_profline()
             self._print_source(proflist)
@@ -433,7 +508,8 @@ class Converter:
             self.prof.print_stats()
 
     def _hookhtml(self):
-        """hook html print"""
+        """hook html print
+        """
         if not os.path.exists(self.output_dir):
             os.makedirs(self.output_dir)
         filepath = "%s/%s" % (self.output_dir, self.output_htmlfile)
@@ -454,14 +530,13 @@ class Converter:
         self.outfile.close()
 
     def printout(self, filetype='html'):
-        """print to html or text."""
+        """print to html or text.
+        """
         self.outputtype = filetype
-        hookpoint = None
         if filetype == 'raw':
-            hookpoint = self._hookraw
+            self._hookraw()
         else:
-            hookpoint = self._hookhtml
-        hookpoint()
+            self._hookhtml()
 
 
 def main():
@@ -497,6 +572,7 @@ def main():
     p2h.output_dir = opts.output_dir
     p2h.printout(outmode)
     if outmode == 'html':
+        p2h = Converter(args[0])
         p2h.functions_number = 99999
         p2h.output_htmlfile = 'index-all.html'
         p2h.printout(outmode)
